// // #define halt(x)  while (1)

#define DEBUG 1
#define DEBUGP(x)  if (DEBUG) {putstring_nl(x);}

#define TRUE 1
#define FALSE 0

#define REGION_US 0
#define REGION_EU 1

// date format
#define DATE 0  // mm-dd-yy
#define DAY 1   // thur jan 1

#define DISPLAYSIZE 9

#define MAXSNOOZE 600 // 10 minute
#define MAXALARMTIMER 60 // Alarm will go off for max 60 minutes
#define INACTIVITYTIMEOUT 10 // how many seconds we will wait before turning off menus

// Frequency for ambilight PWM (use 121 here, it gives 31250 / 3  / 121 = 90 Hz Speed)
// Remember : PWM_MAX cannot be greater than PWM_DIVIDER for software reasons in this case, so if you set the frequency higher (PWM_DIVIDER lower), you'll lose pwm steps.

#define PWM_DIVIDER 121

// Min and max for ambilight PWM (0=LED off, PWM_DIVIDER = LED on)
// You have to change the pwm_sequence in ambilight.c, if you change the number of pwm steps, because its optimized for 120 pwm steps.
#define PWM_MIN 1
#define PWM_MAX 121

#define EE_YEAR 1
#define EE_MONTH 2
#define EE_DAY 3
#define EE_HOUR 4
#define EE_MIN 5
#define EE_SEC 6
#define EE_ALARM_HOUR 7 
#define EE_ALARM_MIN 8
#define EE_BRIGHT 9
#define EE_VOLUME 10
#define EE_REGION 11
#define EE_ALARM_STATE 12
#define EE_EFFECT 13
#define EE_LIGHT 14
#define EE_SNOOZE 15
#define EE_ZONE 16
#define EE_DST 17

// displaymode
#define NONE 99
#define SHOW_TIME 0
#define SHOW_DATE 1
#define SHOW_ALARM 2
#define SHOW_MESSAGE 12
#define SET_TIME 3
#define SET_ALARM 4
#define SET_DATE 5
#define SET_BRIGHTNESS 6
#define SET_VOLUME 7
#define SET_REGION 8
#define SHOW_SNOOZE 9
#define SET_SNOOZE 10
#define SET_TIMER 11
#define SET_EFFECT 12
#define SET_TICK 13
#define SET_LIGHT 14
#define SET_DAYSAVETIME 15

// sub-mode settings
#define SHOW_MENU 0
// alarm/time
#define SET_HOUR 1
#define SET_MIN 2
#define SET_SEC 3
// date
#define SET_MONTH 1
#define SET_DAY 2
#define SET_YEAR 3
//brightness
#define SET_BRITE 1
//volume
#define SET_VOL 1
//region
#define SET_REG 1
//timer
#define SET_TIM 1

//efect
#define SET_DST 1 

//efect
#define SET_EFF 1

//light
#define SET_LIG 1

//ticktack
#define SET_TIC 1

// Temperature compensation values
#define SENSORNULL 645		// Temp sensor 0 degrees - examples in my case 645 for wood clock, 650 for desk clock
#define SENSOR100  850		// Temp sensor 100 degrees
#define SENSOROFFSET 0

#define BOOST PD6
#define BOOST_DDR DDRD
#define BOOST_PORT PORTD

#define BUTTON1 PD5
#define BUTTON2 PB0

//enable this and disable other line for original state
//#define BUTTON3 PD1
#define BUTTON3 PD4

#define VFDSWITCH PD3
#define VFDSWITCH_DDR DDRD
#define VFDSWITCH_PORT PORTD
#define VFDCLK PB5
#define VFDCLK_DDR DDRB
#define VFDCLK_PORT PORTB
#define VFDDATA PB3
#define VFDDATA_DDR DDRB
#define VFDDATA_PORT PORTB
#define VFDLOAD PC0
#define VFDLOAD_DDR DDRC
#define VFDLOAD_PORT PORTC
#define VFDBLANK PC3
#define VFDBLANK_DDR DDRC
#define VFDBLANK_PORT PORTC

#define DCF PD2
#define DCF_DDR DDRD
#define DCF_PORT PORTD
#define DCF_PIN PIND

#define SPK1 PB1
#define SPK2 PB2
#define SPK_PORT PORTB
#define SPK_DDR DDRB

#define DCF1 PC5
#define LED1 PC5 //white
#define LED2 PC4 //red
#define LED3 PD0 //Gruen
#define LED4 PC1 //blue

#define SEG_A 19
#define SEG_B 17
#define SEG_C 14
#define SEG_D 13
#define SEG_E 15
#define SEG_F 18
#define SEG_G 16
#define SEG_H 11

#define DIG_1 4
#define DIG_2 12
#define DIG_3 5
#define DIG_4 10
#define DIG_5 6
#define DIG_6 9
#define DIG_7 8
#define DIG_8 7
#define DIG_9 3

#define nop asm("nop")

//Functions

void delay(uint16_t delay);

void (*app_start)(void) = 0x0000;

//ambilight.c
void ambilight_pwm(void);
void ambilight_pwm_calc(void);
void recreate_pwm_sequence(void);

//dcf.c
void dcf_counter_reset(void);
void dcf_date_reset(void);
void dcf_time_reset(void);
void dcf_store_check(void);
void dcf_0(void);
void dcf_1(void);
void dcf_match_date(void);
void dcf_match_time(void);

//display.c
void control_display(void);
void display_time(uint8_t h, uint8_t m, uint8_t s, uint8_t float_in);
void display_date(uint8_t d, uint8_t m, uint8_t y, uint8_t style, uint8_t float_in);
void display_alarm(uint8_t h, uint8_t m);
void display_countdown(uint8_t c, uint8_t float_in);
void display_str(char *s, uint8_t float_in);
void display_temperature(uint8_t float_in);
void float_display(void);
void create_shadow(void);
void setdisplay(uint8_t digit, uint8_t segments);
void vfd_send(uint32_t d);
void spi_xfer(uint8_t c);

//init.c
void adc_init(void);
void basic_init(void);
void button_init(void);
void boost_init(void);
void clock_init(void);
void speaker_init(void);
void vfd_init(void);

//interrupt.c
void button_press_13(void);
void button_press_2(void);
void dcf_analysis(void);
void set_standby(void);
void time_calc(void);
void update_display(void);

//menu.c
void show_menu(void);
void set_alarm(void);
void set_countdown(void);
void set_time(void);
void set_date(void);
void set_dst(void);
void set_display_effect(void);
void set_ambilight(void);
void set_brightness(void);
void set_volume(void);
void set_region(void);

//temperature.c 
void read_temperature (void);

//util.c
void alarm_off(void);
void beep(uint16_t freq, uint8_t times);
void calc_timezone(void);
uint8_t calc_dotw(void); 
void delayms(uint16_t ms);
void gotosleep(void);
void kickthedog(void);
uint8_t leapyear(uint16_t y);
void setboost (uint8_t brightness);
void setalarmstate(uint8_t alarm_on);
void setsnooze(void);
void tick(void);

//only 39 Bytes of ram until here

//Interrupt Handler

ISR(ANALOG_COMP_vect)
{
set_standby();
}

ISR(INT0_vect)
{
dcf_analysis();
}

ISR(PCINT2_vect)
{
button_press_13();
}

ISR(PCINT0_vect)
{
button_press_2();
}

ISR(TIMER0_OVF_vect)
{
update_display();
}

ISR (TIMER2_OVF_vect)
{
time_calc();
}

//Global Variables

//main control structure
//this is a structure to save RAM
//it took only 8 bit (1 Byte) of ram, instead of 64 bit (8 Byte) for each variable independent

typedef struct
{
    uint8_t alarm_on		:1;	// is the alarm on?
    uint8_t alarming		:1;	// is the alarm currently active?
    uint8_t brightness          :7;      // brightness
    uint8_t countdown_is_set	:1;	// is the countdown-timer on?
    uint8_t countdown_alarming	:1;	// is the countdown alarm currently active?
    uint8_t dcf_sync            :1;     // dcf sync?
    uint8_t dst                 :1;     // summer/wintertime switch enabled?
    uint8_t restored		:1;	// restored?
    uint8_t sleepmode		:1;	// are we in low power sleep mode?
    uint8_t timeunknown		:1;	// is time maybe wrong?
    uint8_t volume		:1;	// how loud is the speaker
}
control_struct;

control_struct ctrl_struct;

//Time structure to store current time and date
//It uses 36 Bits (40 Bits/5 Byte) to store time
//If we store time in separate uint8_t, we need 8*8 = 64 Bit (8 Byte) to do this.

typedef struct
{
    uint8_t s	:6;	// seconds = 6 bit
    uint8_t m	:6;	// minute = 6 bit 
    uint8_t h	:5;	// hour = 5 bit
    uint8_t d	:6;	// day = 6 bit
    uint8_t mo	:4;	// month = 4 bit
    uint8_t y	:7;	// year = 7 bit
    uint8_t z	:1;	// summer/wintertime = 1bit
    uint8_t r	:1;	// region US/EU
}
time_struct;

time_struct time;

//DCF data structure

typedef struct
{
    uint16_t counter_high    :16;
    uint16_t counter_low     :16;
    uint8_t bit_number       :6;

    uint8_t week_day         :4;
    uint8_t day              :6;
    uint8_t month            :4;
    uint8_t year             :7;
    uint8_t day_check        :6;
    uint8_t month_check      :4;
    uint8_t year_check       :7;
    uint8_t date_parity      :5;

    uint8_t minute           :6;
    uint8_t minute_parity    :4;
    uint8_t hour             :5;
    uint8_t minute_check     :6;
    uint8_t hour_check       :5;
    uint8_t hour_parity      :4;
    uint8_t timezone         :1;
}
radio_struct;

radio_struct dcf_struct; 

// effect control
// this is a structure to save RAM
// It took only 8 Bit (1 Byte) of RAM instead of 8*8 = 64 Bit (8 Byte)

typedef struct
{
    uint8_t beeping		:1;	//alarm/timer beeping 1 Hz on/off
    uint8_t create_shadow_en	:1; 	//control for shadow effect
    uint8_t date       		:1; 	//effect control date
    uint8_t effect		:1;	//display effects on/off
    uint8_t light		:1;	//ambilight on/pulse
    uint8_t message_timer 	:1;  	//effect control message/timer
    uint8_t temp  		:1;	//effect control temperature 
    uint8_t time  		:1;	//effect control time 
}
effekt_struct;

effekt_struct effect_ctrl;

typedef struct
{
    uint8_t red	:1;	// dim up/down red
    uint8_t green	:1;	// dim up/down green
    uint8_t blue	:1;	// dim up/down blue
}
pwm_sequence_struct;

pwm_sequence_struct pwm_sequence;

// whether the alarm is on, going off, and alarm time
volatile uint8_t alarm_h, alarm_m, alarmtimer;
//how long have we been snoozing?
uint16_t snoozetimer = 0;

// what is being displayed on the screen? (eg time, date, menu...)
volatile uint8_t displaymode;

// Our display buffer, which is updated to show the time/date/etc
// and is multiplexed onto the tube
uint8_t display[DISPLAYSIZE+2], display2[DISPLAYSIZE+2], display3[DISPLAYSIZE+2]; // stores segments, not values!
uint8_t currdigit = 0;        // which digit we are currently multiplexing

// This table allow us to index between what digit we want to light up
// and what the pin number is on the MAX6921 see the .h for values.
// Stored in ROM (PROGMEM) to save RAM
const char digittable[] PROGMEM = {
  DIG_9, DIG_8, DIG_7, DIG_6, DIG_5, DIG_4, DIG_3, DIG_2, DIG_1
};
PGM_P const digittable_p PROGMEM = digittable;

// This table allow us to index between what segment we want to light up
// and what the pin number is on the MAX6921 see the .h for values.
// Stored in ROM (PROGMEM) to save RAM
const char segmenttable[] PROGMEM = {
  SEG_H, SEG_G,  SEG_F,  SEG_E,  SEG_D,  SEG_C,  SEG_B,  SEG_A 
};
PGM_P const segmenttable_p PROGMEM = segmenttable;

// muxdiv and MUX_DIVIDER divides down a high speed interrupt (31.25KHz)
// down so that we can refresh at about 100Hz (31.25KHz / 300)
// We refresh the entire display at 100Hz so each digit is updated
// 100Hz/DISPLAYSIZE
uint16_t muxdiv = 0;
#define MUX_DIVIDER (300 / DISPLAYSIZE)

// Likewise divides 100Hz * DISPLAYSIZE down to 1Hz for the alarm beeping
uint16_t alarmdiv = 0;
#define ALARM_DIVIDER (100 * DISPLAYSIZE)

// We have a non-blocking delay function, milliseconds is updated by
// an interrupt
volatile uint16_t milliseconds = 0;

// We use the pin change interrupts to detect when buttons are pressed

// These store the current button states for all 3 buttons. We can 
// then query whether the buttons are pressed and released or pressed
// This allows for 'high speed incrementing' when setting the time
volatile uint8_t last_buttonstate = 0, just_pressed = 0, pressed = 0;
volatile uint8_t buttonholdcounter = 0;

// This variable keeps track of whether we have not pressed any
// buttons in a few seconds, and turns off the menu display
volatile uint8_t timeoutcounter = 0;

// Timer for countdown
uint8_t countdown = 0;

// Overblending control
uint8_t overblend = 0;

// Global ambilight PWM variables used to synchronize ambilight pwm sequence with the clock 
uint8_t pwm_blue_count,pwm_red_count,pwm_green_count,pwm_blue,pwm_red,pwm_green,pwmdiv;

//temperature

uint32_t temperature = 0;
uint8_t rgb_temperature = 0; 

//total amount for all variables is apx 120 Byte!

